﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DrawImageTransform
{
    public partial class Form1 : Form
    {
        private Bitmap bm = Properties.Resources.tulips;
        private PointF center;
        private float zoom;
        private float angle;

        public Form1()
        {
            InitializeComponent();
            Reset();

            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.DoubleBuffer, true);
            SetStyle(ControlStyles.UserPaint, true);
        }

        void Reset()
        {
            center.X = ClientRectangle.X + ClientRectangle.Width/2;
            center.Y = ClientRectangle.Y + ClientRectangle.Height/2;
            zoom = 1f;
            angle = 0f;
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.TranslateTransform(center.X, center.Y);
            e.Graphics.ScaleTransform(zoom, zoom);
            e.Graphics.RotateTransform(angle);

            // 이미지
            e.Graphics.DrawImage(bm, -bm.Width/4, -bm.Height/4, 400, 300);

            // 텍스트
            Font F = new Font("맑은 고딕", 12);
            e.Graphics.DrawString("마우스 왼쪽버튼 : 중앙점 이동", F, Brushes.Black, -150, -160);
            e.Graphics.DrawString("마우스 휠 또는 위/아래 키 : 확대/축소", F, Brushes.Red, -150, -140);
            e.Graphics.DrawString("오른쪽/왼쪽 키 : 회전", F, Brushes.Green, -150, -120);
            e.Graphics.DrawString("스페이스바 : Reset", F, Brushes.Blue, -150, -100);
            e.Graphics.DrawRectangle(new Pen(Color.Cyan, 3), -170, -170, 340, 100);
        }

        private void Form1_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                center.X = e.X;
                center.Y = e.Y;
                Invalidate();
            }
        }

        private void Form1_MouseWheel(object sender, MouseEventArgs e)
        {
            if (e.Delta > 0)
            {
                zoom += 0.1f;
            }
            else
            {
                if (zoom > 0.1f) zoom -= 0.1f;
            }
            Invalidate();
        }

        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Space:
                    Reset();
                    break;
                case Keys.Right:
                    angle -= 5;
                    break;
                case Keys.Left:
                    angle += 5;
                    break;
                case Keys.Up:
                    zoom += 0.1f;
                    break;
                case Keys.Down:
                    if (zoom > 0.1f) zoom -= 0.1f;
                    break;
            }
            Invalidate();
        }
    }
}